@@ -1,6 +1,6 @@
 package Beam::Wire;
 # ABSTRACT: Lightweight Dependency Injection Container
-$Beam::Wire::VERSION = '1.012';
+$Beam::Wire::VERSION = '1.013';
 use strict;
 use warnings;
 
@@ -159,11 +159,11 @@ sub fix_refs {
 }
 
 sub parse_args {
-    my ( $self, $class, $args ) = @_;
+    my ( $self, $for, $class, $args ) = @_;
     return if not $args;
     my @args;
     if ( ref $args eq 'ARRAY' ) {
-        @args = $self->find_refs( @{$args} );
+        @args = $self->find_refs( $for, @{$args} );
     }
     elsif ( ref $args eq 'HASH' ) {
         # Hash args could be a ref
@@ -176,13 +176,13 @@ sub parse_args {
             if ( exists $args{file} && !path( $args{file} )->is_absolute ) {
                 $args{file} = $self->dir->child( $args{file} );
             }
-            @args = $self->find_refs( %args );
+            @args = $self->find_refs( $for, %args );
             if ( $config ) {
                 push @args, config => $config;
             }
         }
         else {
-            my ( $maybe_ref ) = $self->find_refs( $args );
+            my ( $maybe_ref ) = $self->find_refs( $for, $args );
             if ( blessed $maybe_ref ) {
                 @args = ( $maybe_ref );
             }
@@ -237,7 +237,7 @@ sub create_service {
             my $method_name = $m->{method};
             my $return = $m->{return} || q{};
             delete $service_info{args};
-            my @args = $self->parse_args( $service_info{class}, $m->{args} );
+            my @args = $self->parse_args( $name, $service_info{class}, $m->{args} );
             my $invocant = $service || $service_info{class};
             my $output = $invocant->$method_name( @args );
             $service = !$service || $return eq 'chain' ? $output
@@ -245,10 +245,15 @@ sub create_service {
         }
     }
     else {
-        my @args = $self->parse_args( @service_info{"class","args"} );
+        my @args = $self->parse_args( $name, @service_info{"class","args"} );
         $service = $service_info{class}->$method( @args );
     }
 
+    if ( my $with = $service_info{with} ) {
+        my @roles = ref $with ? @{ $with } : ( $with );
+        Moo::Role->apply_roles_to_object( $service, @roles );
+    }
+
     if ( $service_info{on} ) {
         my %meta = $self->get_meta_names;
         my @listeners;
@@ -275,7 +280,7 @@ sub create_service {
                 _deprecated( 'warning: (deprecated) "$method" in event handlers is now "$sub" in service "' . $name . '"' );
             }
             my $sub_name = delete $conf->{ $meta{sub} } || delete $conf->{ $meta{method} };
-            my ( $listen_svc ) = $self->find_refs( $conf );
+            my ( $listen_svc ) = $self->find_refs( $name, $conf );
             $service->on( $event => sub { $listen_svc->$sub_name( @_ ) } );
         }
     }
@@ -308,14 +313,14 @@ sub merge_config {
 }
 
 sub find_refs {
-    my ( $self, @args ) = @_;
+    my ( $self, $for, @args ) = @_;
     my @out;
     my %meta = $self->get_meta_names;
     for my $arg ( @args ) {
         if ( ref $arg eq 'HASH' ) {
             if ( $self->is_meta( $arg ) ) {
                 if ( $arg->{ $meta{ ref } } ) {
-                    push @out, $self->resolve_ref( $arg );
+                    push @out, $self->resolve_ref( $for, $arg );
                 }
                 else { # Try to treat it as a service to create
                     my %service_info;
@@ -329,11 +334,11 @@ sub find_refs {
                 }
             }
             else {
-                push @out, { $self->find_refs( %{$arg} ) };
+                push @out, { $self->find_refs( $for, %{$arg} ) };
             }
         }
         elsif ( ref $arg eq 'ARRAY' ) {
-            push @out, [ map { $self->find_refs( $_ ) } @{$arg} ];
+            push @out, [ map { $self->find_refs( $for, $_ ) } @{$arg} ];
         }
         else {
             push @out, $arg; # simple scalars
@@ -359,12 +364,13 @@ sub get_meta_names {
         class   => "${prefix}class",
         extends => "${prefix}extends",
         sub     => "${prefix}sub",
+        call    => "${prefix}call",
     );
     return wantarray ? %meta : \%meta;
 }
 
 sub resolve_ref {
-    my ( $self, $arg ) = @_;
+    my ( $self, $for, $arg ) = @_;
 
     my %meta = $self->get_meta_names;
 
@@ -377,7 +383,24 @@ sub resolve_ref {
         my $conf = $self->get_config($name);
         @ref = dpath( $path )->match($service);
     }
+    elsif ( my $call = $arg->{ $meta{call} } ) {
+        my ( $method, @args );
+
+        if ( ref $call eq 'HASH' ) {
+            $method = $call->{ $meta{method} };
+            my $args = $call->{ $meta{args} };
+            @args = !$args ? ()
+                  : ref $args eq 'ARRAY'  ? @{ $args }
+                  : $args;
+        }
+        else {
+            $method = $call;
+        }
+
+        @ref = $service->$method( @args );
+    }
     elsif ( my $method = $arg->{ $meta{method} } ) {
+        _deprecated( 'warning: (deprecated) Using "$method" to get a value in a dependency is now "$call" in service "' . $for . '"' );
         my $args = $arg->{ $meta{args} };
         my @args = !$args                ? ()
                  : ref $args eq 'ARRAY'  ? @{ $args }
@@ -433,7 +456,7 @@ sub _load_config {
 
 
 package Beam::Wire::Exception;
-$Beam::Wire::Exception::VERSION = '1.012';
+$Beam::Wire::Exception::VERSION = '1.013';
 use Moo;
 with 'Throwable';
 use Types::Standard qw( :all );
@@ -446,7 +469,7 @@ has error => (
 
 
 package Beam::Wire::Exception::Constructor;
-$Beam::Wire::Exception::Constructor::VERSION = '1.012';
+$Beam::Wire::Exception::Constructor::VERSION = '1.013';
 use Moo;
 use Types::Standard qw( :all );
 extends 'Beam::Wire::Exception';
@@ -459,7 +482,7 @@ has attr => (
 
 
 package Beam::Wire::Exception::Service;
-$Beam::Wire::Exception::Service::VERSION = '1.012';
+$Beam::Wire::Exception::Service::VERSION = '1.013';
 use Moo;
 use Types::Standard qw( :all );
 extends 'Beam::Wire::Exception';
@@ -477,7 +500,7 @@ has file => (
 
 
 package Beam::Wire::Exception::NotFound;
-$Beam::Wire::Exception::NotFound::VERSION = '1.012';
+$Beam::Wire::Exception::NotFound::VERSION = '1.013';
 use Moo;
 extends 'Beam::Wire::Exception::Service';
 
@@ -493,7 +516,7 @@ has '+error' => (
 
 
 package Beam::Wire::Exception::InvalidConfig;
-$Beam::Wire::Exception::InvalidConfig::VERSION = '1.012';
+$Beam::Wire::Exception::InvalidConfig::VERSION = '1.013';
 use Moo;
 extends 'Beam::Wire::Exception::Service';
 use overload q{""} => sub {
@@ -519,7 +542,7 @@ Beam::Wire - Lightweight Dependency Injection Container
 
 =head1 VERSION
 
-version 1.012
+version 1.013
 
 =head1 SYNOPSIS
 
@@ -724,6 +747,25 @@ Hash C<args> will be merged seperately, like so:
 C<activemq_dev> will get the C<port>, C<user>, and C<password> arguments
 from the base service C<activemq>.
 
+=head4 with
+
+Compose roles into the service object.
+
+    app:
+        class: My::App
+        with: My::FeatureRole
+
+    otherapp:
+        class: My::App
+        with:
+            - My::FeatureRole
+            - My::OtherFeatureRole
+
+This lets you break features out into roles, and compose those roles a la carte
+on the fly. If you have 20 different optional features, it is difficult to create
+every possible combination of them. So, C<with> allows you to pick the features
+you want.
+
 =head4 lifecycle
 
 Control how your service is created. The default value, C<singleton>, will cache
@@ -763,12 +805,12 @@ Attach event listeners using L<Beam::Emitter|Beam::Emitter>.
         on:
             before_my_event:
                 $ref: listener
-                $method: on_before_my_event
+                $sub: on_before_my_event
             my_event:
                 - $ref: listener
-                  $method: on_my_event
+                  $sub: on_my_event
                 - $ref: other_listener
-                  $method: on_my_event
+                  $sub: on_my_event
     listener:
         class: My::Listener
     other_listener:
@@ -785,13 +827,13 @@ as an array of hashes:
         on:
             - before_my_event:
                 $ref: listener
-                $method: on_before_my_event
+                $sub: on_before_my_event
             - my_event:
                 $ref: listener
-                $method: on_my_event
+                $sub: on_my_event
             - my_event:
                 $ref: other_listener
-                $method: on_my_event
+                $sub: on_my_event
 
 =head3 Config Services
 
